# -*- coding: utf-8 -*-
"""
Created on Sat Aug  6 23:30:41 2022

@author: m.alfouly
"""
from hisim import log
import numpy as np

# owned
from hisim import component as cp
from hisim.simulationparameters import SimulationParameters
from hisim.loadtypes import LoadTypes, Units
from hisim.components.weather import Weather
from hisim.components.building import Building
from hisim.components.PIDcontroller import PIDController
import hisim.utils as utils


class AirConditionerState:
    """
    This data class saves the state of the air conditioner
    """

    def __init__(self, timestep_actual: int = -1, state: int = 0, timestep_of_last_action: int = 0):
        self.timestep_actual = timestep_actual
        self.state = state
        self.timestep_of_last_action = timestep_of_last_action

    def clone(self):
        return AirConditionerState(timestep_actual=self.timestep_actual, state=self.state,
                                   timestep_of_last_action=self.timestep_of_last_action)

    def is_first_iteration(self, timestep):
        if self.timestep_actual + 1 == timestep:
            self.timestep_actual += 1
            return True
        else:
            return False

    def activation(self, timestep):
        self.state = 1
        self.timestep_of_last_action = timestep

    def deactivation(self, timestep):
        self.state = 0
        self.timestep_of_last_action = timestep


class AirConditioner(cp.Component):
    # inputs
    State = "State"
    TemperatureOutside = "TemperatureOutside"
    ElectricityOutputPID = "ElectricityOutputPID"

    # outputs
    ThermalEnergyDelivered = "ThermalEnergyDelivered"
    ElectricityOutput = "ElectricityOutput"

    def __init__(self,
                 my_simulation_parameters: SimulationParameters,
                 manufacturer: str = "Panasonic",
                 name: str = "CS-RE18JKE/CU-RE18JKE",
                 min_operation_time: int = 60 * 60,
                 min_idle_time: int = 15 * 60,
                 control: str = "on_off"):
        super().__init__("AirConditioner", my_simulation_parameters=my_simulation_parameters)
        self.build(manufacturer, name, min_operation_time, min_idle_time)

        self.t_outC: cp.ComponentInput = self.add_input(self.component_name,
                                                        self.TemperatureOutside,
                                                        LoadTypes.TEMPERATURE,
                                                        Units.CELSIUS,
                                                        True)
        self.stateC: cp.ComponentInput = self.add_input(self.component_name,
                                                        self.State,
                                                        LoadTypes.ANY,
                                                        Units.ANY,
                                                        False)
        self.electric_power: cp.ComponentInput = self.add_input(self.component_name,
                                                                self.ElectricityOutputPID,
                                                                LoadTypes.ELECTRICITY,
                                                                Units.WATT,
                                                                False)

        self.thermal_energy_deliveredC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                             self.ThermalEnergyDelivered,
                                                                             LoadTypes.HEATING,
                                                                             Units.WATT)
        self.electricity_outputC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                       self.ElectricityOutput,
                                                                       LoadTypes.ELECTRICITY,
                                                                       Units.WATT)

        self.add_default_connections(Weather, self.get_weather_default_connections())
        self.add_default_connections(AirConditionercontroller, self.get_controller_default_connections())

        self.control = control

    def get_weather_default_connections(self):
        print("setting weather default connections")
        connections = []
        weather_classname = Weather.get_classname()
        connections.append(
            cp.ComponentConnection(AirConditioner.TemperatureOutside, weather_classname, Weather.TemperatureOutside))
        return connections

    def get_controller_default_connections(self):
        log.information("setting controller default connections in AirConditioner")
        connections = []
        controller_classname = AirConditionercontroller.get_classname()
        connections.append(
            cp.ComponentConnection(AirConditioner.State, controller_classname, AirConditionercontroller.State))
        return connections

    def build(self, manufacturer, name, min_operation_time, min_idle_time):
        # Simulation parameters

        # Retrieves air conditioner from database - BEGIN
        air_conditioners_database = utils.load_smart_appliance("Air Conditioner")

        air_conditioner = None
        for air_conditioner_iterator in air_conditioners_database:
            if air_conditioner_iterator["Manufacturer"] == manufacturer and air_conditioner_iterator["Model"] == name:
                air_conditioner = air_conditioner_iterator
                break

        if air_conditioner is None:
            raise Exception("Air conditioner model not registered in the database")

        self.manufacturer = manufacturer
        self.model = name
        # Interpolates COP, cooling capacities, power input data from the database
        self.eer_ref = []
        self.t_out_cooling_ref = []
        self.t_out_heating_ref = []
        self.cooling_capacity_ref = []
        self.heating_capacity_ref = []
        self.cop_ref = []

        """
        Typical realtion between COPs and Heating capacities are found here:  https://www.everysolarthing.com/blog/heat-pumps/

        """

        for air_conditioner_tout in air_conditioner['Outdoor temperature range - cooling']:
            self.t_out_cooling_ref.append([air_conditioner_tout][0])
        for air_conditioner_tout in air_conditioner['Outdoor temperature range - heating']:
            self.t_out_heating_ref.append([air_conditioner_tout][0])

        for air_conditioner_eers in air_conditioner['EER W/W']:
            self.eer_ref.append([air_conditioner_eers][0])
        for air_conditioner_cops in air_conditioner['COP W/W']:
            self.cop_ref.append([air_conditioner_cops][0])

        for air_conditioner_cooling_capacities in air_conditioner['Cooling capacity W']:
            self.cooling_capacity_ref.append([air_conditioner_cooling_capacities][0])
        for air_conditioner_heating_capacities in air_conditioner['Heating capacity W']:
            self.heating_capacity_ref.append([air_conditioner_heating_capacities][0])
        print(str(self.t_out_cooling_ref))
        print(str(self.eer_ref))
        self.eer_coef = np.polyfit(self.t_out_cooling_ref, self.eer_ref, 1)
        self.cooling_capacity_coef = np.polyfit(self.t_out_cooling_ref, self.cooling_capacity_ref, 1)

        self.cop_coef = np.polyfit(self.t_out_heating_ref, self.cop_ref, 1)
        self.heating_capacity_coef = np.polyfit(self.t_out_heating_ref, self.heating_capacity_ref, 1)

        # Retrieves air conditioner from database - END

        # Sets the time operation restricitions
        self.on_time = min_operation_time / self.my_simulation_parameters.seconds_per_timestep
        self.off_time = min_idle_time / self.my_simulation_parameters.seconds_per_timestep

        self.state = AirConditionerState()
        self.previous_state = AirConditionerState()

    def cal_eer(self, t_out):
        return np.polyval(self.eer_coef, t_out)

    def cal_cooling_capacity(self, t_out):
        return np.polyval(self.cooling_capacity_coef, t_out)

    def cal_cop(self, t_out):
        return np.polyval(self.cop_coef, t_out)

    def cal_heating_capacity(self, t_out):
        return np.polyval(self.heating_capacity_coef, t_out)

    def i_save_state(self) -> None:
        self.previous_state = self.state.clone()
        pass

    def i_restore_state(self) -> None:
        self.state = self.previous_state.clone()
        pass

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues) -> None:
        pass

    def write_to_report(self):
        lines = []
        lines.append("Air conditioner")
        lines.append("Air conditioner manufacturer {}".format(self.manufacturer))
        lines.append("Air conditioner model {}".format(self.model))
        return lines

    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues, force_convergence: bool) -> None:
        # Inputs
        t_out = stsv.get_input_value(self.t_outC)
        on_off_state = stsv.get_input_value(self.stateC)

        if self.control == "on_off":
            # Heating Season:
            cop = self.cal_cop(t_out)
            heating_power = self.cal_heating_capacity(t_out)

            # Cooling Season:
            eer = self.cal_eer(t_out)
            cooling_power = self.cal_cooling_capacity(t_out)

            # save reference state state0 in first iteration
            if self.state.is_first_iteration(timestep):
                self.state0 = self.state.clone()

            # return device on if minimum operation time is not fulfilled and device was on in previous state
            if (self.state0.state == 1 and self.state0.timestep_of_last_action + self.on_time >= timestep):
                self.state.state = 1
            elif (self.state0.state == -1 and self.state0.timestep_of_last_action + self.on_time >= timestep):
                self.state.state = -1
            # return device off if minimum idle time is not fulfilled and device was off in previous state
            elif (self.state0.state == 0 and self.state0.timestep_of_last_action + self.off_time >= timestep):
                self.state.state = 0
            # check signal from l2 and turn on or off if it is necesary
            else:
                if on_off_state == 0 and (self.state0.state == 1 or self.state0.state == -1):
                    self.state.deactivation(timestep)
                elif (on_off_state == 1 or on_off_state == -1) and self.state0.state == 0:
                    self.state.activation(timestep)

            if self.state.state == 1 and on_off_state == 1:
                thermal_energy_delivered = on_off_state * heating_power
                electricity_output = heating_power / cop
            elif self.state.state == 1 and on_off_state == -1:
                thermal_energy_delivered = on_off_state * cooling_power
                electricity_output = cooling_power / eer
            else:
                thermal_energy_delivered = 0
                electricity_output = 0

            # log.information("thermal_energy_delivered {}".format(thermal_energy_delivered))
            stsv.set_output_value(self.thermal_energy_deliveredC, thermal_energy_delivered)
            stsv.set_output_value(self.electricity_outputC, electricity_output)

        if self.control == "PID":
            Electric_Power = stsv.get_input_value(self.electric_power)
            if Electric_Power > 0:
                cop = self.cal_cop(t_out)
                thermal_energy_delivered = Electric_Power * cop
            elif Electric_Power < 0:
                eer = self.cal_eer(t_out)
                thermal_energy_delivered = Electric_Power * eer
            else:
                thermal_energy_delivered = 0

            stsv.set_output_value(self.thermal_energy_deliveredC, thermal_energy_delivered)

            # self.state = AirConditionerState(thermal_energy_delivered = thermal_energy_delivered)
            # stsv.set_output_value(self.copC, cop)


class AirConditionercontroller(cp.Component):
    """
    Air Conditioner Controller. It takes data from other
    components and sends signal to the air conditioner for
    activation or deactivation.

    Parameters
    --------------
    t_air_cooling: float
        Maximum comfortable temperature for residents
    offset: float
        Temperature offset to compensate the hysteresis
        correction for the building temperature change
    mode : int
        Mode index for operation type for this air conditioner
    """
    # Inputs
    TemperatureMean = "Residence Temperature"
    ElectricityInput = "ElectricityInput"

    # Outputs
    State = "State"

    # Similar components to connect to:
    # 1. Building
    @utils.measure_execution_time
    def __init__(self,
                 my_simulation_parameters: SimulationParameters,
                 t_air_heating: float = 18.0,
                 t_air_cooling: float = 26.0,
                 offset: float = 0.0
                 ):
        super().__init__("AirConditionercontroller", my_simulation_parameters=my_simulation_parameters)
        self.build(t_air_cooling=t_air_cooling, t_air_heating=t_air_heating, offset=offset)

        self.t_mC: cp.ComponentInput = self.add_input(self.component_name,
                                                      self.TemperatureMean,
                                                      LoadTypes.TEMPERATURE,
                                                      Units.CELSIUS,
                                                      True)
        self.electricity_inputC: cp.ComponentInput = self.add_input(self.component_name,
                                                                    self.ElectricityInput,
                                                                    LoadTypes.ELECTRICITY,
                                                                    Units.WATT,
                                                                    False)
        self.stateC: cp.ComponentOutput = self.add_output(self.component_name,
                                                          self.State,
                                                          LoadTypes.ANY,
                                                          Units.ANY)

        self.add_default_connections(Building, self.get_building_default_connections())

    def get_building_default_connections(self):
        log.information("setting building default connections in AirConditionercontroller")
        connections = []
        building_classname = Building.get_classname()
        connections.append(cp.ComponentConnection(AirConditionercontroller.TemperatureMean, building_classname,
                                                  Building.TemperatureMean))
        return connections

    def build(self, t_air_heating, t_air_cooling, offset):
        # Sth
        self.controller_ACmode = "off"
        self.previous_AC_mode = self.controller_ACmode

        # Configuration
        self.t_set_heating = t_air_heating
        self.t_set_cooling = t_air_cooling
        self.offset = offset

    def i_save_state(self):
        self.previous_AC_mode = self.controller_ACmode

    def i_restore_state(self):
        self.controller_ACmode = self.previous_AC_mode

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues) -> None:
        pass

    def write_to_report(self):
        lines = []
        lines.append("Air Conditioner Controller")
        lines.append("Control algorith of the Air conditioner is: on-off control\n")
        lines.append("Controller heating set temperature is {} Deg C \n".format(self.t_set_heating))
        lines.append("Controller cooling set temperature is {} Deg C \n".format(self.t_set_cooling))
        return lines

    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues, force_convergence: bool) -> None:
        # check demand, and change state of self.has_heating_demand, and self._has_cooling_demand
        if force_convergence:
            pass
        else:
            # Retrieves inputs
            t_m_old = stsv.get_input_value(self.t_mC)
            # electricity_input = stsv.get_input_value(self.electricity_inputC)

            self.conditions(t_m_old)

        if self.controller_ACmode == 'heating':
            state = 1
        if self.controller_ACmode == 'cooling':
            state = -1
        if self.controller_ACmode == 'off':
            state = 0

        stsv.set_output_value(self.stateC, state)
        # log.information("state {}".format(state))

    def conditions(self, set_temp):
        maximum_heating_set_temp = self.t_set_heating + self.offset
        minimum_heating_set_temp = self.t_set_heating
        minimum_cooling_set_temp = self.t_set_cooling - self.offset
        maximum_cooling_set_temp = self.t_set_cooling

        if self.controller_ACmode == 'heating':  # and daily_avg_temp < 15:
            if set_temp > maximum_heating_set_temp:  # 16.5
                self.controller_ACmode = 'off'
                return
        if self.controller_ACmode == 'cooling':
            if set_temp < minimum_cooling_set_temp:  # 23.5
                self.controller_ACmode = 'off'
                return
        if self.controller_ACmode == 'off':
            # if pvs_surplus > ? and air_temp < minimum_heating_air + 2:
            if set_temp < 16:  # 21
                self.controller_ACmode = 'heating'
                return
            if set_temp > 24:  # 26
                self.controller_ACmode = 'cooling'
                return

    def prin1t_outpu1t(self, t_m, state):
        log.information("==========================================")
        log.information("T m: {}".format(t_m))
        log.information("State: {}".format(state))



# -*- coding: utf-8 -*-
"""
Created on Tue Jul  5 12:39:29 2022

@author: Johanna
"""

# Owned
from typing import List, Any
from hisim.simulationparameters import SimulationParameters

from dataclasses import dataclass
from dataclasses_json import dataclass_json
from hisim import component as cp
from hisim import loadtypes as lt

from hisim.components.configuration import PhysicsConfig
from hisim.components import generic_hydrogen_storage
from hisim import log
from hisim import utils
__authors__ = "Frank Burkrad, Maximilian Hillen"
__copyright__ = "Copyright 2021, the House Infrastructure Project"
__credits__ = ["Noah Pflugradt"]
__license__ = ""
__version__ = ""
__maintainer__ = "Johanna Ganglbauer"
__email__ = "johanna.ganglbauer@4wardenergy.at"
__status__ = ""
#

@dataclass_json
@dataclass
class GenericElectrolyzerConfig:
    name: str
    source_weight: int
    min_power: float # [W]
    max_power: float # [W]
    min_hydrogen_production_rate_hour: float # [Nl/h]
    max_hydrogen_production_rate_hour: float # [Nl/h]
    
    def init( self, name: str,
                    source_weight: int,
                    min_power: float,
                    max_power: float,
                    min_hydrogen_production_rate_hour: float,
                    max_hydrogen_production_rate_hour: float ) -> None:
        self.name = name
        self.source_weight = source_weight
        self.min_power = min_power
        self.max_powwer = max_power
        self.min_hydrogen_production_rate_hour = min_hydrogen_production_rate_hour
        self.max_hydrogen_production_rate_hour = max_hydrogen_production_rate_hour
        
    @staticmethod
    def get_default_config() -> Any:
        config = GenericElectrolyzerConfig(name ="Electrolyzer",
                                           source_weight = 1,
                                           min_power = 1200,  # [W]
                                           max_power = 2400,  # [W]
                                           min_hydrogen_production_rate_hour = 300,  # [Nl/h]
                                           max_hydrogen_production_rate_hour = 5000  # [Nl/h]
                                           )
        return config
        
class ElectrolyzerState:
    """
    This data class saves the state of the electrolyzer.
    """

    def __init__( self, hydrogen : float = 0, electricity : float = 0 ):
        self.hydrogen = hydrogen
        self.electricity = electricity
        
    def clone( self ) -> Any:
        return ElectrolyzerState( hydrogen = self.hydrogen, electricity = self.electricity )
        
    
class GenericElectrolyzer(cp.Component):
    ElectricityTarget = "ElectricityTarget"
    HydrogenOutput = "HydrogenOutput"
    
    ElectricityOutput = "ElectricityOutput"   
    
    def __init__(self, my_simulation_parameters : SimulationParameters, config : GenericElectrolyzerConfig):
        """
        The electrolyzer converts electrical energy [kWh] into hydrogen [kg]
        It can work in a certain range from x to 100% or be switched off = 0%
        The conversion rate is given by the supplier and is directly used
            maybe a change to efficiency can be made but its just making things more complex with no benefit
        Between the given values, the values are calculated by an interpolation.
            --> If the load curve is linear a fixed factor could be calculated.

        Therefore it has an operational state
        All the min values and  all the max values are connected and the electrolyzer can operate between them.

        The waste energy in electolyzers is not used to provide heat for the households demand
        Output pressure may be used in the future for the
        """
        
        super().__init__(name=config.name + '_w' + str(config.source_weight), 
                         my_simulation_parameters=my_simulation_parameters )
        self.build( config )
        self.min_hydrogen_production_rate: float
        self.ElectricityTargetC: cp.ComponentInput = self.add_input(self.component_name,
                                                                    GenericElectrolyzer.ElectricityTarget,
                                                                    lt.LoadTypes.ELECTRICITY,
                                                                    lt.Units.WATT,
                                                                    True)
        self.HydrogenOutputC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                   GenericElectrolyzer.HydrogenOutput,
                                                                   lt.LoadTypes.HYDROGEN,
                                                                   lt.Units.KG_PER_SEC)
        self.ElectricityOutputC: cp.ComponentOutput = self.add_output(
            object_name=self.component_name, field_name=GenericElectrolyzer.ElectricityOutput, load_type=lt.LoadTypes.ELECTRICITY,
            unit=lt.Units.WATT, postprocessing_flag=[lt.InandOutputType.ELECTRICITY_CONSUMPTION_EMS_CONTROLLED, lt.ComponentType.ELECTROLYZER])
        self.add_default_connections(L1GenericElectrolyzerController, self.get_l1_controller_default_connections())

    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    def get_l1_controller_default_connections( self ) -> List[cp.ComponentConnection]:
        log.information("setting l1 default connections in generic electrolyzer" )
        connections: List[cp.ComponentConnection] = [ ]
        controller_classname = L1GenericElectrolyzerController.get_classname()
        connections.append(cp.ComponentConnection(GenericElectrolyzer.ElectricityTarget, controller_classname, L1GenericElectrolyzerController.ElectricityTarget))
        return connections
    
    def build(self, config: GenericElectrolyzerConfig) -> None:
        self.state = ElectrolyzerState( )
        self.previous_state = ElectrolyzerState( )
        
        self.name = config.name
        self.source_weight = config.source_weight
        self.min_power = config.min_power
        self.max_power = config.max_power
        self.min_hydrogen_production_rate = config.min_hydrogen_production_rate_hour / 3600
        self.max_hydrogen_production_rate = config.max_hydrogen_production_rate_hour / 3600
        
    def i_save_state(self) -> None:
        self.previous_state = self.state.clone( )

    def i_restore_state(self) -> None:
        self.state = self.previous_state.clone( )
        
    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues) -> None:
        pass
    
    def i_simulate( self, timestep: int, stsv: cp.SingleTimeStepValues,  force_convergence: bool )  -> None:
        # check demand, and change state of self.has_heating_demand, and self._has_cooling_demand
        if force_convergence:
            pass
        
        electricity_target = stsv.get_input_value( self.ElectricityTargetC )
        if electricity_target < 0:
            raise ValueError( 'Target electricity needs to be positive in Electrolyzer' )
        elif 0 <= electricity_target < self.min_power:
            self.state.electricity = 0
            electricity_target=0
        elif electricity_target > self.max_power:
            self.state.electricity = self.max_power
        else:
            self.state.electricity = electricity_target

        #interpolation between points
        # Nl / s
        hydrogen_output_liter: float
        if electricity_target == 0:
            hydrogen_output_liter = 0
        else:
            hydrogen_output_liter = self.min_hydrogen_production_rate \
                                    + ( ( self.max_hydrogen_production_rate - self.min_hydrogen_production_rate )
                                        *  ( self.state.electricity - self.min_power ) / ( self.max_power - self.min_power ) )
        self.state.hydrogen = ( hydrogen_output_liter / 1000 ) * PhysicsConfig.hydrogen_density
        stsv.set_output_value( self.HydrogenOutputC, self.state.hydrogen  )
        stsv.set_output_value( self.ElectricityOutputC, self.state.electricity )
        
    def write_to_report( self ):
        lines = []
        lines.append("Name: {}".format( self.name + str( self.source_weight ) ) )
        lines.append( "electrical power: {:4.0f} kW".format( self.max_power ) )
        lines.append( "hydrogen production rate: {:4.0f} kg / s".format( self.max_hydrogen_production_rate ) )
        return lines
        
@dataclass_json
@dataclass
class L1ElectrolyzerConfig:
    """
    L1Electrolyzer Config
    """
    name : str
    source_weight : int
    min_operation_time : int
    min_idle_time : int
    P_min_electrolyzer : float
    SOC_max_H2 : float

    def __init__( self,
                  name : str,
                  source_weight : int,
                  min_operation_time : int,
                  min_idle_time : int,
                  P_min_electrolyzer : float,
                  SOC_max_H2 : float )  -> None:
        self.name = name
        self.source_weight = source_weight
        self.min_operation_time = min_operation_time
        self.min_idle_time = min_idle_time
        self.P_min_electrolyzer = P_min_electrolyzer
        self.SOC_max_H2 = SOC_max_H2
        
    @staticmethod
    def get_default_config() -> Any:
        config = L1ElectrolyzerConfig( name = 'L1ElectrolyzerRuntimeController',
                                       source_weight =  1,
                                       min_operation_time = 14400,
                                       min_idle_time = 7200,
                                       P_min_electrolyzer = 1200,
                                       SOC_max_H2 = 96 )
        return config
        
class L1ElectrolyzerControllerState:
    """
    This data class saves the state of the controller.
    """

    def __init__( self, timestep_actual : int = -1, state : int = 0, timestep_of_last_action : int = 0 ) -> None:
        self.timestep_actual = timestep_actual
        self.state = state
        self.timestep_of_last_action = timestep_of_last_action
        
    def clone( self )  -> Any:
        return L1ElectrolyzerControllerState(timestep_actual = self.timestep_actual, state = self.state, timestep_of_last_action = self.timestep_of_last_action)
    
    def is_first_iteration( self, timestep: int ) -> bool:
        if self.timestep_actual + 1 == timestep:
            self.timestep_actual += 1
            return True
        else:
            return False
        
    def activation( self, timestep: int ) -> None:
        self.state = 1
        self.timestep_of_last_action = timestep
        
    def deactivation( self, timestep: int ) -> None:
        self.state = 0
        self.timestep_of_last_action = timestep 

class L1GenericElectrolyzerController(cp.Component):
    
    """
    L1 CHP Controller. It takes care of the operation of the CHP only in terms of running times.

    Parameters
    --------------
    min_running_time: int, optional
        Minimal running time of device, in seconds. The default is 3600 seconds.
    min_idle_time : int, optional
        Minimal off time of device, in seconds. The default is 900 seconds.
    source_weight : int, optional
        Weight of component, relevant if there is more than one component of same type, defines hierachy in control. The default is 1.
    component type : str, optional
        Name of component to be controlled
    """
    # Inputs
    l2_ElectricityTarget = "l2_ElectricityTarget"
    HydrogenSOC = "HydrogenSOC"

    # Outputs
    ElectricityTarget = "ElectricityTarget"
    
    # Similar components to connect to:
    # 1. Building
    @utils.measure_execution_time
    def __init__( self, 
                  my_simulation_parameters : SimulationParameters,
                  config : L1ElectrolyzerConfig ) -> None:
        
        super().__init__(name=config.name + '_w' + str(config.source_weight), 
                          my_simulation_parameters = my_simulation_parameters )
        
        self.build( config )
        
        #add inputs
        self.l2_ElectricityTargetC : cp.ComponentInput = self.add_input(self.component_name,
                                                                        self.l2_ElectricityTarget,
                                                                        lt.LoadTypes.ELECTRICITY,
                                                                        lt.Units.WATT,
                                                                        mandatory = True)
        self.HydrogenSOCC : cp.ComponentInput = self.add_input(self.component_name,
                                                               self.HydrogenSOC,
                                                               lt.LoadTypes.HYDROGEN,
                                                               lt.Units.PERCENT,
                                                               mandatory = True)
        #add outputs
        self.ElectricityTargetC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                      self.ElectricityTarget,
                                                                      lt.LoadTypes.ELECTRICITY,
                                                                      lt.Units.WATT)
        
        self.add_default_connections(generic_hydrogen_storage.GenericHydrogenStorage, self.get_hydrogenstorage_default_connections())
        
    def get_hydrogenstorage_default_connections( self ) -> List[cp.ComponentConnection]:
        log.information("setting generic H2 storage default connections in L1 of generic electrolyzer" )
        connections: List[cp.ComponentConnection] = [ ]
        h2storage_classname = generic_hydrogen_storage.GenericHydrogenStorage.get_classname()
        connections.append(cp.ComponentConnection(L1GenericElectrolyzerController.HydrogenSOC, h2storage_classname, generic_hydrogen_storage.GenericHydrogenStorage.HydrogenSOC))
        return connections
    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    def build( self, config: L1ElectrolyzerConfig ) -> None:
        
        self.on_time = int( config.min_operation_time / self.my_simulation_parameters.seconds_per_timestep )
        self.off_time = int( config.min_idle_time / self.my_simulation_parameters.seconds_per_timestep )
        self.name = config.name
        self.source_weight = config.source_weight
        self.Pmin = config.P_min_electrolyzer
        self.SOCmax = config.SOC_max_H2
        
        self.state0 = L1ElectrolyzerControllerState()
        self.state = L1ElectrolyzerControllerState()
        self.previous_state = L1ElectrolyzerControllerState()

    def i_save_state(self) -> None:
        self.previous_state = self.state.clone( )

    def i_restore_state(self) -> None:
        self.state = self.previous_state.clone( )

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues) -> None:
        pass

    def i_simulate( self, timestep: int, stsv: cp.SingleTimeStepValues,  force_convergence: bool ) -> None:
        
        l2_electricity_target = stsv.get_input_value( self.l2_ElectricityTargetC )
        H2_SOC = stsv.get_input_value( self.HydrogenSOCC )
        
        #save reference state state0 in first iteration
        if self.state.is_first_iteration( timestep ):
            self.state0 = self.state.clone( )
            
        #return device on if minimum operation time is not fulfilled and device was on in previous state
        if ( self.state0.state == 1 and self.state0.timestep_of_last_action + self.on_time >= timestep ):
            self.state.state = 1
            l2_electricity_target = max( self.Pmin, l2_electricity_target )   
        #return device off if minimum idle time is not fulfilled and device was off in previous state
        elif ( self.state0.state == 0 and self.state0.timestep_of_last_action + self.off_time >= timestep ):
            self.state.state = 0
            l2_electricity_target = 0
        
        #catch cases where hydrogen storage is close to maximum level and signals oscillate -> just turn off electrolyzer
        elif force_convergence:
            if self.state0.state == 0:
                self.state.state = 0
            else:
                self.state.deactivation( timestep )
            l2_electricity_target = 0
            pass
        
        #set point control
        else:  
            #if target electricity is too low or hydrogen storage too full: turn off
            if ( ( l2_electricity_target < self.Pmin ) or ( H2_SOC > self.SOCmax ) ) and self.state0.state == 1:
                self.state.deactivation( timestep )
            #turns on if electricity is high enough and there is still space in storage, only works if being off is not compulsory
            elif ( ( l2_electricity_target >= self.Pmin ) and ( H2_SOC <= self.SOCmax ) ) and self.state0.state == 0:
                self.state.activation( timestep )
            else:
                pass
                
        stsv.set_output_value( self.ElectricityTargetC, self.state.state * l2_electricity_target )

    def prin1t_outpu1t(self, t_m: float, state: L1ElectrolyzerControllerState) -> None:
        log.information("==========================================")
        log.information("T m: {}".format(t_m))
        log.information("State: {}".format(state))

    def write_to_report(self) -> List[str]:
        lines: List[str] = []
        lines.append("L1 Controller Electrolyzer: " + self.component_name)
        return lines


""" Defines the simulation parameters class. This defines how the simulation will proceed. """
# clean
from __future__ import annotations
from typing import List, Optional
import datetime
from dataclasses import dataclass
from dataclass_wizard import JSONWizard

from hisim import log
from hisim.postprocessingoptions import PostProcessingOptions

from building_sizer.system_config import SystemConfig


@dataclass()
class SimulationParameters(JSONWizard):

    """ Defines HOW the simulation is going to proceed: Time resolution, time span and all these things. """

    start_date: datetime.datetime
    end_date: datetime.datetime
    seconds_per_timestep: int
    post_processing_options: List[int]
    logging_level: int
    result_directory: str
    skip_finished_results: bool
    system_config: SystemConfig

    def __init__(self, start_date: datetime.datetime, end_date: datetime.datetime, seconds_per_timestep: int,
                 result_directory: str = "",
                 post_processing_options: Optional[List[int]] = None, logging_level: int = log.LogPrio.INFORMATION,
                 skip_finished_results: bool = False, system_config: SystemConfig = SystemConfig()):
        """ Initializes the class. """
        self.start_date: datetime.datetime = start_date
        self.end_date: datetime.datetime = end_date
        self.seconds_per_timestep = seconds_per_timestep
        self.duration = end_date - start_date
        total_seconds = self.duration.total_seconds()
        self.timesteps: int = int(total_seconds / seconds_per_timestep)
        self.year: int = int(start_date.year)
        if post_processing_options is None:
            post_processing_options = []
        self.post_processing_options: List[int] = post_processing_options
        self.logging_level: int = logging_level  # Info # noqa
        self.result_directory: str = result_directory
        self.skip_finished_results: bool = skip_finished_results
        self.system_config = system_config  # noqa

    @classmethod
    def full_year(cls, year: int, seconds_per_timestep: int) -> SimulationParameters:
        """ Generates a parameter set for a full year without any post processing, primarily for unit testing. """
        return cls(datetime.datetime(year, 1, 1), datetime.datetime(year + 1, 1, 1), seconds_per_timestep, "")

    def enable_all_options(self) -> None:
        """ Enables all the post processing options . """
        for option in PostProcessingOptions:
            self.post_processing_options.append(option)

    @classmethod
    def full_year_all_options(cls, year: int, seconds_per_timestep: int) -> SimulationParameters:
        """ Generates a parameter set for a full year with all the post processing, primarily for unit testing. """
        pars = cls(datetime.datetime(year, 1, 1), datetime.datetime(year + 1, 1, 1), seconds_per_timestep, "")
        pars.enable_all_options()
        return pars

    @classmethod
    def january_only(cls, year: int, seconds_per_timestep: int) -> SimulationParameters:
        """ Generates a parameter set for a single january, primarily for unit testing. """
        return cls(datetime.datetime(year, 1, 1), datetime.datetime(year, 1, 31), seconds_per_timestep, "")

    @classmethod
    def three_months_only(cls, year: int, seconds_per_timestep: int) -> SimulationParameters:
        """ Generates a parameter set for a single january, primarily for unit testing. """
        return cls(datetime.datetime(year, 1, 1), datetime.datetime(year, 6, 30), seconds_per_timestep, "")

    @classmethod
    def one_week_only(cls, year: int, seconds_per_timestep: int) -> SimulationParameters:
        """ Generates a parameter set for a single week, primarily for unit testing. """
        return cls(datetime.datetime(year, 1, 1), datetime.datetime(year, 1, 8), seconds_per_timestep, "")

    @classmethod
    def one_day_only(cls, year: int, seconds_per_timestep: int = 60) -> SimulationParameters:
        """ Generates a parameter set for a single day, primarily for unit testing. """
        return cls(datetime.datetime(year, 1, 1), datetime.datetime(year, 1, 2), seconds_per_timestep, "")

    @classmethod
    def one_day_only_with_all_options(cls, year: int, seconds_per_timestep: int) -> SimulationParameters:
        """ Generates a parameter set for a single day, primarily for unit testing. """
        pars = cls(datetime.datetime(year, 1, 1), datetime.datetime(year, 1, 2), seconds_per_timestep, "")
        pars.enable_all_options()
        return pars

    def get_unique_key(self) -> str:
        """ Gets a unique key from a simulation parameter class. """
        return str(self.start_date) + "###" + str(self.end_date) + "###" + str(self.seconds_per_timestep) + "###" + str(
            self.year) + "###" + str(self.timesteps)



# Generic/Built-in
import copy

# Owned
from hisim import component as cp
from hisim import loadtypes as lt
from hisim import utils
from hisim.components.generic_ev_charger  import SimpleStorageState
from hisim.simulationparameters import SimulationParameters
__authors__ = "Vitor Hugo Bellotto Zago"
__copyright__ = "Copyright 2021, the House Infrastructure Project"
__credits__ = ["Noah Pflugradt"]
__license__ = "MIT"
__version__ = "0.1"
__maintainer__ = "Vitor Hugo Bellotto Zago"
__email__ = "vitor.zago@rwth-aachen.de"
__status__ = "development"


class GenericBatteryState:
    def __init__(self, init_stored_energy=0, max_stored_energy=None, min_stored_energy=None, max_var_stored_energy=None, min_var_stored_energy=None):
        self.stored_energy = init_stored_energy
        self.max_stored_energy = max_stored_energy
        self.min_stored_energy = min_stored_energy
        self.max_var_stored_energy = max_var_stored_energy
        self.min_var_stored_energy = min_var_stored_energy

    def charge(self, energy):
        energy = abs(energy)
        if self.stored_energy + energy < self.max_stored_energy:
            charge = energy
        else:
            charge = self.max_stored_energy - self.stored_energy

        if charge > self.max_var_stored_energy:
            charge = self.max_var_stored_energy

        self.stored_energy = charge + self.stored_energy
        self.chargeWh = charge

    def discharge(self, energy):
        energy = - abs(energy)
        if self.stored_energy + energy > self.min_stored_energy:
            discharge = energy
        else:
            discharge = self.stored_energy - self.min_stored_energy

        if discharge < self.min_var_stored_energy:
            discharge = self.min_var_stored_energy

        self.stored_energy = discharge + self.stored_energy
        self.chargeWh = discharge


class GenericBattery(cp.Component):
    # Imports
    ElectricityInput = "ElectricityInput"
    State = "State"

    # Outputs
    StoredEnergy = "StoredEnergy"
    StateOfCharge = "StateOfCharge"
    ElectricityOutput = "ElectricityOutput"

    def __init__(self,
                 my_simulation_parameters: SimulationParameters,
                 manufacturer:str="sonnen",
                 model:str="sonnenBatterie 10 - 11,5 kWh",
                 soc:float=10/15,
                 base:bool=False)->None:
        super().__init__("Battery", my_simulation_parameters)

        self.build(manufacturer=manufacturer, model=model, base=base)

        self.state = SimpleStorageState(max_var_val=self.max_var_stored_energy,
                                        min_var_val=self.min_var_stored_energy,
                                        stored_energy=self.max_stored_energy*soc)
        self.previous_state = copy.deepcopy(self.state)

        self.inputC : cp.ComponentInput = self.add_input(self.component_name,
                                                         self.ElectricityInput,
                                                         lt.LoadTypes.ELECTRICITY,
                                                         lt.Units.WATT,
                                                         True)
        self.stateC : cp.ComponentInput = self.add_input(self.component_name,
                                                         self.State,
                                                         lt.LoadTypes.ANY,
                                                         lt.Units.ANY,
                                                         True)

        self.state_of_chargeC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                    self.StateOfCharge,
                                                                    lt.LoadTypes.ANY,
                                                                    lt.Units.ANY)

        self.stored_energyC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                  self.StoredEnergy,
                                                                  lt.LoadTypes.ELECTRICITY,
                                                                  lt.Units.WATT_HOUR)

        self.electricity_outputC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                       self.ElectricityOutput,
                                                                       lt.LoadTypes.ELECTRICITY,
                                                                       lt.Units.WATT)

    def build(self, manufacturer, model, base):
        self.base = base
        self.time_correction_factor = 1 / self.my_simulation_parameters.seconds_per_timestep
        self.seconds_per_timestep = self.my_simulation_parameters.seconds_per_timestep

        # Gets flexibilities, including heat pump
        battery_database = utils.load_smart_appliance("Battery")

        battery_found = False
        for battery in battery_database:
            if battery["Manufacturer"] == manufacturer and battery["Model"] == model:
                battery_found = True
                break

        if battery_found == False:
            raise Exception("Heat pump model not registered in the database")

        self.max_stored_energy = battery['Capacity'] * 1E3
        self.min_stored_energy = self.max_stored_energy * 0.0
        self.efficiency = battery['Efficiency']
        self.efficiency_inverter = battery['Inverter Efficiency']
        self.max_var_stored_energy = battery['Maximal Charging Power'] * 1E3 * self.time_correction_factor
        if 'Maximal Discharging Power' in battery:
            self.min_var_stored_energy = - battery['Maximal Discharging Power'] * 1E3 * self.time_correction_factor
        else:
            self.min_var_stored_energy = - self.max_var_stored_energy

    def write_to_report(self):
        lines =[]
        lines.append("MaxStoredEnergy: {}".format(self.max_stored_energy))
        return lines

    #def i_save_state(self):
    #    self.previous_state = copy.copy(self.state)

    #def i_restore_state(self):
    #    self.state = copy.copy(self.previous_state)
    def i_save_state(self)-> None:
        self.previous_state = copy.deepcopy(self.state)
    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    def i_restore_state(self)-> None:
        self.state = copy.deepcopy(self.previous_state)

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues) -> None:
        pass

    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues, force_convergence: bool)-> None:
        load = stsv.get_input_value(self.inputC)
        state = stsv.get_input_value(self.stateC)

        load = - load / self.seconds_per_timestep

        capacity = self.state.stored_energy
        max_capacity = self.max_stored_energy
        min_capacity = self.min_stored_energy

        if state == 1:
            charging_delta, after_capacity = self.state.store(max_capacity=max_capacity,
                                                              current_capacity=capacity,
                                                              val=load,
                                                              efficiency=self.efficiency_inverter)
        elif state == -1:
            charging_delta, after_capacity = self.state.withdraw(min_capacity=min_capacity,
                                                                 current_capacity=capacity,
                                                                 val=load,
                                                                 efficiency=self.efficiency_inverter*self.efficiency)
        else:
            charging_delta = 0
            after_capacity = capacity

        stsv.set_output_value(self.state_of_chargeC, after_capacity/max_capacity)
        stsv.set_output_value(self.stored_energyC, after_capacity)
        stsv.set_output_value(self.electricity_outputC, charging_delta)

class BatteryController(cp.Component):
    ElectricityInput = "ElectricityInput"
    State = "State"

    def __init__(self, my_simulation_parameters: SimulationParameters )-> None:
        super().__init__(name="BatteryController", my_simulation_parameters=my_simulation_parameters)

        self.inputC : cp.ComponentInput = self.add_input(self.component_name,
                                                         self.ElectricityInput,
                                                         lt.LoadTypes.ELECTRICITY,
                                                         lt.Units.WATT,
                                                         True)
        self.stateC : cp.ComponentOutput = self.add_output(self.component_name,
                                                           self.State,
                                                           lt.LoadTypes.ANY,
                                                           lt.Units.ANY)

    def i_save_state(self)-> None:
        pass

    def i_restore_state(self)-> None:
        pass
    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues)-> None:
        pass

    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues,  force_convergence: bool)-> None:
        load = stsv.get_input_value(self.inputC)
        state:float = 0
        if load < 0.0:
            state = 1
        elif load > 0.0:
            state = -1
        else:
            state = 0.0

        stsv.set_output_value(self.stateC, state)







# Import packages from standard library or the environment e.g. pandas, numpy etc.
from dataclasses import dataclass
from dataclasses_json import dataclass_json
from typing import List, Any
# Import modules from HiSim
from hisim import component as cp
from hisim import loadtypes as lt
from hisim.simulationparameters import SimulationParameters
from hisim.components.building import Building
from hisim.components import controller_l1_heatpump
from hisim import log
__authors__ = "Johanna Ganglbauer - johanna.ganglbauer@4wardenergy.at"
__copyright__ = "Copyright 2021, the House Infrastructure Project"
__credits__ = ["Noah Pflugradt"]
__license__ = "MIT"
__version__ = "0.1"
__maintainer__ = "Vitor Hugo Bellotto Zago"
__email__ = "vitor.zago@rwth-aachen.de"
__status__ = "development"

@dataclass_json
@dataclass
class HeatSourceConfig:
    """
    HeatSource Config
    """
    name: str
    source_weight: int
    fuel : lt.LoadTypes
    power_th : float
    efficiency : float

    def __init__(self,
                 name: str,
                 source_weight: int,
                 fuel: lt.LoadTypes,
                 power_th: float,
                 efficiency: float) -> None:
        self.name = name
        self.source_weight = source_weight
        self.fuel = fuel
        self.power_th = power_th
        self.efficiency = efficiency
        
    @staticmethod
    def get_default_config_heating() -> Any:
        config = HeatSourceConfig(name='HeatingHeatSource', source_weight=1, fuel=lt.LoadTypes.DISTRICTHEATING, 
                                  power_th = 6200, efficiency=1.0) 
        return config
    
    @staticmethod
    def get_default_config_waterheating() -> Any:
        config = HeatSourceConfig(name='DHWHeatSource', source_weight=1, fuel=lt.LoadTypes.DISTRICTHEATING,
                                  power_th = 3000, efficiency=1.0 ) 
        return config
            
class HeatSourceState:
    """
    This data class saves the state of the CHP.
    """

    def __init__( self, state : int = 0 ):
        self.state = state
        
    def clone( self ):
        return HeatSourceState( state = self.state )

class HeatSource( cp.Component ):
    """
    District heating implementation. District Heating transmitts heat with given efficiency.
    District heating is controlled with a HeatSourceTargetPercentage control oscillating within the comfort temperature band.
    """
    
    # Inputs
    L1HeatSourceTargetPercentage = "L1HeatSourceTargetPercentage"

    # Outputs
    ThermalPowerDelivered = "ThermalPowerDelivered"
    FuelDelivered = "FuelDelivered"

    def __init__(self, my_simulation_parameters: SimulationParameters, config: HeatSourceConfig) -> None:
        """
        Parameters
        ----------
        max_power : int
            Maximum power of district heating.
        efficiency : float
            Efficiency of heat transfer
        """
        
        super( ).__init__(config.name + '_w' + str(config.source_weight), my_simulation_parameters = my_simulation_parameters)
        
        #introduce parameters of district heating
        self.name = config.name
        self.source_weight = config.source_weight
        self.fuel = config.fuel
        self.power_th = config.power_th
        self.efficiency = config.efficiency
        self.state = HeatSourceState( )
        self.previous_state = HeatSourceState( )

        # Inputs - Mandatories
        self.l1_heatsource_taget_percentage: cp.ComponentInput = self.add_input(
            self.component_name, self.L1HeatSourceTargetPercentage, lt.LoadTypes.ANY, lt.Units.PERCENT, mandatory=True)

        # Outputs 
        self.ThermalPowerDeliveredC : cp.ComponentOutput = self.add_output( 
            object_name=self.component_name, field_name=self.ThermalPowerDelivered, load_type=lt.LoadTypes.HEATING,
            unit=lt.Units.WATT, postprocessing_flag=[lt.InandOutputType.THERMAL_PRODUCTION])
        self.FuelDeliveredC: cp.ComponentOutput = self.add_output(
            object_name=self.component_name, field_name=self.FuelDelivered, load_type=self.fuel,
            unit=lt.Units.ANY, postprocessing_flag=[lt.InandOutputType.FUEL_CONSUMPTION])
        
        if config.fuel == lt.LoadTypes.OIL:
            self.FuelDeliveredC.unit = lt.Units.LITER        
        else:
            self.FuelDeliveredC.unit = lt.Units.WATT_HOUR
        
        self.add_default_connections(controller_l1_heatpump.L1HeatPumpController, self.get_l1_controller_default_connections())
        
    def get_l1_controller_default_connections(self) -> List[cp.ComponentConnection]:
        log.information("setting l1 default connections in Generic Heat Source")
        connections = [ ]
        controller_classname = controller_l1_heatpump.L1HeatPumpController.get_classname()
        connections.append(cp.ComponentConnection(HeatSource.L1HeatSourceTargetPercentage, controller_classname,
                                                  controller_l1_heatpump.L1HeatPumpController.HeatControllerTargetPercentage))
        return connections

    def write_to_report( self ) -> List[str]:
        """
        Returns
        -------
        lines : list of strings
            Text to enter report.
        """
        
        lines = []
        lines.append( "Name: {}".format( self.name + str( self.source_weight ) ) )
        lines.append( "Fuel: {}".format( self.fuel ) )
        lines.append( "Power: {:4.0f} kW".format( ( self.power_th ) * 1E-3 ) )
        lines.append( 'Efficiency : {:4.0f} %'.format( ( self.efficiency ) * 100 ) )
        return lines
    
    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    
    def i_save_state(self) -> None:
        self.previous_state = self.state.clone( )

    def i_restore_state(self)  -> None:
        self.state = self.previous_state.clone( )

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues)  -> None:
        pass
    
    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues,  force_convergence: bool)  -> None:
        """
        Performs the simulation of the district heating model.
        """ 
        
        # Inputs
        target_percentage = stsv.get_input_value(self.l1_heatsource_taget_percentage)

        # calculate modulation
        if target_percentage > 0:
            power_modifier = target_percentage
        if target_percentage == 0:
            power_modifier = 0
        if target_percentage < 0:
            power_modifier = 0
        if power_modifier > 1:
            power_modifier = 1

        stsv.set_output_value(self.ThermalPowerDeliveredC, self.power_th * power_modifier * self.efficiency)

        if self.fuel == lt.LoadTypes.OIL:
            #conversion from Wh oil to liter oil
            stsv.set_output_value(self.FuelDeliveredC, power_modifier * self.power_th * 1.0526315789474e-4 * self.my_simulation_parameters.seconds_per_timestep / 3.6e3)
        else:
            stsv.set_output_value(self.FuelDeliveredC, power_modifier * self.power_th * self.my_simulation_parameters.seconds_per_timestep / 3.6e3)


# Generic/Built-in
import copy

# Owned
from hisim.component import Component, SingleTimeStepValues, ComponentInput, ComponentOutput
from hisim import loadtypes as lt
from hisim.simulationparameters import SimulationParameters
__authors__ = "Vitor Hugo Bellotto Zago"
__copyright__ = "Copyright 2021, the House Infrastructure Project"
__credits__ = ["Noah Pflugradt"]
__license__ = "MIT"
__version__ = "0.1"
__maintainer__ = "Vitor Hugo Bellotto Zago"
__email__ = "vitor.zago@rwth-aachen.de"
__status__ = "development"

class SimpleController(Component):
    StorageFillLevel = "Fill Level Percent"
    GasHeaterPowerPercent = "Gas Heater Power Level"

    def __init__(self, name: str, my_simulation_parameters: SimulationParameters ) -> None:
        super().__init__(name, my_simulation_parameters=my_simulation_parameters)
        self.input1: ComponentInput = self.add_input(self.component_name, SimpleController.StorageFillLevel, lt.LoadTypes.ELECTRICITY, lt.Units.KWH, True)
        self.output1: ComponentOutput = self.add_output(self.component_name, SimpleController.GasHeaterPowerPercent, lt.LoadTypes.GAS, lt.Units.PERCENT)
        self.state = 0
        self.previous_state = self.state

    def i_save_state(self) -> None:
        self.previous_state = self.state

    def i_restore_state(self) -> None:
        self.state = self.previous_state

    def i_simulate(self, timestep: int, stsv: SingleTimeStepValues, force_convergence: bool) -> None:
        if force_convergence:
            return
        percent = stsv.get_input_value(self.input1)
        if percent < 0.4:
            self.state = 1
        if percent > 0.99:
            self.state = 0
        stsv.set_output_value(self.output1, self.state)



# Generic/Built-in
import numpy as np
import copy
import matplotlib
import seaborn
from math import pi
from typing import List, Any, Optional
# Owned
import hisim.utils as utils
from hisim.components.weather import Weather
from hisim import component as cp
from hisim.loadtypes import LoadTypes, Units
from hisim.simulationparameters import SimulationParameters
#from hisim.components.extended_storage import WaterSlice
#from hisim.components.configuration import WarmWaterStorageConfig
#from hisim.components.configuration import PhysicsConfig
from hisim.components.building import Building
from hisim.components.weather import Weather
from hisim import log


__authors__ = "Vitor Hugo Bellotto Zago"
__copyright__ = "Copyright 2021, the House Infrastructure Project"
__credits__ = ["Noah Pflugradt"]
__license__ = "MIT"
__version__ = "0.1"
__maintainer__ = "Vitor Hugo Bellotto Zago"
__email__ = "vitor.zago@rwth-aachen.de"
__status__ = "development"

class GenericHeatPumpState:

    def __init__(self,
                 start_timestep: int=0,
                 thermal_energy_delivered: float = 0.0,
                 cop:float = 1.0,
                 cycle_number:Optional[int]=None) -> None:
        self.start_timestep=start_timestep
        self.thermal_energy_delivered = thermal_energy_delivered
        self.cycle_number = cycle_number
        if thermal_energy_delivered == 0.0:
            self.activation = 0
            self.heating = 0.0
            self.cooling = 0.0
            self.cop = 1.0
            self.electricity_in = abs(self.thermal_energy_delivered / self.cop)
        elif self.thermal_energy_delivered > 0.0:
            self.activation = -1
            self.heating = self.thermal_energy_delivered
            self.cooling = 0.0
            self.cop = cop
            self.electricity_in = abs(self.thermal_energy_delivered / self.cop)
        elif self.thermal_energy_delivered < 0.0:
            self.activation = 1
            self.heating = 0
            self.cooling = self.thermal_energy_delivered
            self.cop = cop
            self.electricity_in = abs(self.thermal_energy_delivered / self.cop)
        else:
            raise Exception("Impossible Heat Pump State.")
            
    def clone(self) -> Any:
        return GenericHeatPumpState(self.start_timestep, self.thermal_energy_delivered, self.cop, self.cycle_number)

class GenericHeatPump(cp.Component):
    """
    Heat pump implementation. It does support a
    refrigeration cycle. Thermal output is delivered straight to
    the component object.

    Parameters
    ----------
    manufacturer : str
        Heat pump manufacturer
    name : str
        Heat pump model
    min_operation_time : int, optional
        Minimum time duration that the heat pump operates under one cycle, in seconds. The default is 3600.
    min_idle_time : int, optional
        Minimum time duration that the heat pump has to stay idle, in seconds. The default is 900.
    """
    # Inputs
    State = "State"
    TemperatureOutside = "TemperatureOutside"
    WaterConsumption = "WaterConsumption"
    WaterInput_mass = "WaterInput_mass"                             # kg/s
    WaterInput_temperature = "WaterInput_temperature"               # °C

    # Outputs
    #WaterOutput_mass = "WaterOutput_mass"                           # kg/s
    #WaterOutput_temperature = "WaterOutput_temperature"             # °C
    #WastedEnergyMaxTemperature = "Wasted Energy Max Temperature"    # W

    ThermalEnergyDelivered = "ThermalEnergyDelivered"
    Heating = "Heating"
    Cooling = "Cooling"
    ElectricityOutput = "ElectricityOutput"
    NumberOfCycles = "NumberOfCycles"

    # Similar components to connect to:
    # 1. Weather
    # 2. HeatPumpController
    @utils.measure_execution_time
    def __init__(self,
                 my_simulation_parameters: SimulationParameters,
                 manufacturer : str = "Viessmann Werke GmbH & Co KG",
                 name : str ="Vitocal 300-A AWO-AC 301.B07",
                 min_operation_time : int = 60 * 60,
                 min_idle_time : int = 15 * 60) -> None:
        super().__init__("HeatPump", my_simulation_parameters=my_simulation_parameters)

        self.build(manufacturer, name, min_operation_time, min_idle_time)

        self.number_of_cycles = 0
        self.number_of_cycles_previous = copy.deepcopy(self.number_of_cycles)
        self.state = GenericHeatPumpState(start_timestep=int(0), cycle_number=0)
        self.previous_state = self.state.clone( )



        # Inputs - Mandatories
        self.stateC: cp.ComponentInput = self.add_input(self.component_name,
                                                        self.State,
                                                        LoadTypes.ANY,
                                                        Units.ANY,
                                                        True)
        self.t_outC: cp.ComponentInput = self.add_input(self.component_name,
                                                        self.TemperatureOutside,
                                                        LoadTypes.ANY,
                                                        Units.CELSIUS,
                                                        True)
        # Inputs - Not Mandatories
        self.water_loadC: cp.ComponentInput = self.add_input(self.component_name,
                                                             self.WaterConsumption,
                                                             LoadTypes.VOLUME,
                                                             Units.LITER,
                                                             False)
        self.water_input_mass: cp.ComponentInput = self.add_input(self.component_name,
                                                                  self.WaterInput_mass,
                                                                  LoadTypes.WARM_WATER,
                                                                  Units.KG_PER_SEC,
                                                                  False)
        self.water_input_temperature: cp.ComponentInput = self.add_input(self.component_name,
                                                                         self.WaterInput_temperature,
                                                                         LoadTypes.WARM_WATER,
                                                                         Units.CELSIUS,
                                                                         False)



        # Outputs
        #self.water_output_mass: cp.ComponentOutput = self.add_output(self.ComponentName,
        #                                                          self.WaterOutput_mass,
        #                                                          LoadTypes.WarmWater,
        #                                                          Units.kg_per_sec)
        #self.water_output_temperature: cp.ComponentOutput = self.add_output(self.ComponentName,
        #                                                                 self.WaterOutput_temperature,
        #ä                                                                      LoadTypes.WarmWater,
        #                                                                 Units.Celsius)
        #self.wasted_energy_max_temperature: cp.ComponentOutput = self.add_output(self.ComponentName,
        #                                                                      self.WastedEnergyMaxTemperature,
        #                                                                      LoadTypes.WarmWater,
        #                                                                      Units.Watt)

        self.thermal_energy_deliveredC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                             self.ThermalEnergyDelivered,
                                                                             LoadTypes.HEATING,
                                                                             Units.WATT)

        self.heatingC: cp.ComponentOutput = self.add_output(self.component_name,
                                                            self.Heating,
                                                            LoadTypes.HEATING,
                                                            Units.WATT)

        self.coolingC: cp.ComponentOutput = self.add_output(self.component_name,
                                                            self.Cooling,
                                                            LoadTypes.COOLING,
                                                            Units.WATT)

        self.electricity_outputC: cp.ComponentOutput = self.add_output(self.component_name,
                                                                       self.ElectricityOutput,
                                                                       LoadTypes.ELECTRICITY,
                                                                       Units.WATT)

        self.number_of_cyclesC : cp.ComponentOutput = self.add_output(self.component_name,
                                                                      self.NumberOfCycles,
                                                                      LoadTypes.ANY,
                                                                      Units.ANY)
        
        self.add_default_connections( Weather, self.get_weather_default_connections( ) )
        self.add_default_connections( HeatPumpController, self.get_controller_default_connections( ) )
        
    def get_weather_default_connections( self ) -> List[cp.ComponentConnection]:
        log.information("setting weather default connections in HeatPump")
        connections = [ ]
        weather_classname = Weather.get_classname( )
        connections.append(cp.ComponentConnection(GenericHeatPump.TemperatureOutside, weather_classname, Weather.TemperatureOutside))
        return connections
    
    def get_controller_default_connections( self )  -> List[cp.ComponentConnection]:
        log.information("setting controller default connections in HeatPump")
        connections = [ ]
        controller_classname = HeatPumpController.get_classname( )
        connections.append(cp.ComponentConnection(GenericHeatPump.State, controller_classname, HeatPumpController.State))
        return connections
    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    def build( self, manufacturer: str, name: str, min_operation_time: float, min_idle_time:float )  -> None:
        # Simulation parameters

        # Retrieves heat pump from database - BEGIN
        heat_pumps_database = utils.load_smart_appliance("Heat Pump")

        heat_pump_found = False
        for heat_pump in heat_pumps_database:
            if heat_pump["Manufacturer"] == manufacturer and heat_pump["Name"] == name:
                heat_pump_found = True
                break

        if heat_pump_found == False:
            raise Exception("Heat pump model not registered in the database")

        # Interpolates COP data from the database
        self.cop_ref = []
        self.t_out_ref = []
        for heat_pump_cops in heat_pump['COP']:
            self.t_out_ref.append(float([*heat_pump_cops][0][1:].split("/")[0]))
            self.cop_ref.append(float([*heat_pump_cops.values()][0]))
        self.cop_coef = np.polyfit(self.t_out_ref, self.cop_ref, 1)

        self.max_heating_power = heat_pump['Nominal Heating Power A2/35'] * 1E3
        #self.max_heating_power = 11 * 1E3
        self.max_cooling_power = - self.max_heating_power
        # Retrieves heat pump from database - END

        # Sets the power variation restrictions
        # Default values: 15 minutes to full power
        # Used only for non-clocked heat pump
        self.max_heating_power_var = self.max_heating_power * self.my_simulation_parameters.seconds_per_timestep / 900
        self.max_cooling_power_var = - self.max_heating_power * self.my_simulation_parameters.seconds_per_timestep / 900

        # Sets the time operation restricitions
        self.min_operation_time = min_operation_time / self.my_simulation_parameters.seconds_per_timestep
        self.min_idle_time = min_idle_time / self.my_simulation_parameters.seconds_per_timestep

        # Writes info to report
        self.write_to_report()

        # Applies correction due to timestep
        self.set_time_correction()
        #self.set_time_correction(self.time_correction_factor)

    def set_time_correction(self, factor: float = 1) -> None:
        if factor == 1:
            self.HasBeenConverted = False
        if self.HasBeenConverted is True:
            raise Exception("It has been already converted!")
        self.max_heating_power *= factor
        self.max_cooling_power *= factor
        self.max_heating_power_var *= factor
        self.max_cooling_power_var *= factor
        if factor != 1:
            self.HasBeenConverted = True


    def cal_cop(self, t_out:float) -> float:
        val:float = self.cop_coef[0] * t_out + self.cop_coef[1]
        return val

    def i_save_state(self)->None:
        self.previous_state = self.state.clone()
        self.number_of_cycles_previous = self.number_of_cycles

    def i_restore_state(self) -> None:
        self.state = self.previous_state.clone() # copy.deepcopy(self.previous_state)
        self.number_of_cycles = self.number_of_cycles_previous

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues) -> None:
        pass

    def write_to_report(self) -> List[str]:
        lines: List[str] = []
        lines.append("Name: {}".format("Heat Pump"))
        lines.append("Max power: {:4.0f} kW".format((self.max_heating_power)*1E-3))
        lines.append("Max power var: {:4.0f}".format(self.max_heating_power_var))
        #lines = []
        #lines.append([self.ComponentName,""])
        #lines.append(["Max power:","{:4.2f}".format(self.max_heating_power)])
        #lines.append(["Max power var:","{:4.2f}".format(self.max_heating_power_var)])
        return lines

    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues,  force_convergence: bool) -> None:
        # Inputs
        stateC = stsv.get_input_value(self.stateC)
        t_out = stsv.get_input_value(self.t_outC)
        #log.information("State: {}, Temperature: {}".format(stateC, t_out))
        #log.information("State of Activation: {}".format(self.state.activation))
        #log.information("Timestep special: {}".format(self.state.start_timestep + self.min_idle_time))
        # Calculation

        ## Calculation.ThermalEnergyStorage
        ## ToDo: Implementation with Thermal Energy Storage - BEGIN
        #if self.water_loadC.SourceOutput is not None:
        #    if stsv.get_input_value(self.water_loadC) != 0:
        #        control_signal = 1
        #    else:
        #        control_signal = 0
        #    # Inputs
        #    water_input_mass_sec = stsv.get_input_value(self.water_input_mass)
        #    water_input_mass = water_input_mass_sec
        #    water_input_temperature = stsv.get_input_value(self.water_input_temperature)

        #    mass_flow_max = self.max_heating_power / (4180 * 25)  # kg/s ## -> ~0.07

        #    if control_signal == 1 and (water_input_mass == 0 and water_input_temperature == 0):
        #        """first iteration"""
        #        water_input_temperature = 40
        #        water_input_mass = mass_flow_max

        #    if control_signal == 1:
        #        volume_flow_gasheater = water_input_mass / PhysicsConfig.water_density
        #        ws = WaterSlice(WarmWaterStorageConfig.tank_diameter, (4 * volume_flow_gasheater) / (pi * WarmWaterStorageConfig.tank_diameter ** 2), water_input_temperature)
        #        ws_output, wasted_energy_max_temperature, thermal_output = self.process_thermal(ws)
        #    else:
        #        height_flow_gasheater = 0
        #        water_input_temperature = 0
        #        ws = WaterSlice(WarmWaterStorageConfig.tank_diameter, height_flow_gasheater, water_input_temperature)
        #        ws_output = ws
        #        wasted_energy_max_temperature = 0

        #    ws_output_mass = ws_output.mass
        #    ws_output_temperature = ws_output.temperature

        #    # Mass is consistent
        #    stsv.set_output_value(self.water_output_mass, ws_output_mass)
        #    stsv.set_output_value(self.water_output_temperature, ws_output_temperature)
        #    stsv.set_output_value(self.wasted_energy_max_temperature, wasted_energy_max_temperature)
        ## ToDo: Implementation with Thermal Energy Storage - END


        ## Calculation.ThermalEnergyDelivery
        ### Heat Pump is on
        if self.state.activation != 0:
            number_of_cycles = self.state.cycle_number
            # Checks if the minimum running time has been reached
            if timestep >= self.state.start_timestep + self.min_operation_time and stateC == 0:
                self.state = GenericHeatPumpState(start_timestep=timestep,
                                                  cycle_number=number_of_cycles)

            stsv.set_output_value(self.thermal_energy_deliveredC, self.state.thermal_energy_delivered)
            stsv.set_output_value(self.heatingC, self.state.heating)
            stsv.set_output_value(self.coolingC, self.state.cooling)
            stsv.set_output_value(self.electricity_outputC, self.state.electricity_in)
            stsv.set_output_value(self.number_of_cyclesC, self.number_of_cycles)
            return

        ### Heat Pump is Off
        if stateC != 0 and (timestep >= self.state.start_timestep + self.min_idle_time):
            self.number_of_cycles = self.number_of_cycles + 1
            number_of_cycles = self.number_of_cycles
            if stateC == 1:
            #if stsv.get_input_value(self.stateC) > 0:
                self.state = GenericHeatPumpState(start_timestep=timestep,
                                                  thermal_energy_delivered = self.max_heating_power,
                                                  cop=self.cal_cop(t_out),
                                                  cycle_number=number_of_cycles)
            else:
                self.state = GenericHeatPumpState(start_timestep=timestep,
                                                  thermal_energy_delivered = self.max_cooling_power,
                                                  cop=self.cal_cop(t_out),
                                                  cycle_number = number_of_cycles)

        #log.information(self.state.thermal_energy_delivered)
        # Outputs
        stsv.set_output_value(self.thermal_energy_deliveredC, self.state.thermal_energy_delivered)
        stsv.set_output_value(self.heatingC, self.state.heating)
        stsv.set_output_value(self.coolingC, self.state.cooling)
        stsv.set_output_value(self.electricity_outputC, self.state.electricity_in)
        stsv.set_output_value(self.number_of_cyclesC, self.number_of_cycles)

    def process_thermal(self, ws_in: float) -> None:
        pass
        #temperature_max = 55
        #heat_capacity = PhysicsConfig.water_specific_heat_capacity
        #thermal_energy_to_add = self.max_heating_power
        #ws_out_mass = ws_in.mass
        #try:
        #    ws_out_temperature = ws_in.temperature + thermal_energy_to_add / (heat_capacity * ws_out_mass)
        #except ZeroDivisionError:
        #    log.information(heat_capacity)
        #    log.information(ws_out_mass)
        #    log.information(ws_in.mass)
        #    raise ValueError
        #wasted_energy = 0
        #if ws_out_temperature > temperature_max:
        #    delta_T = ws_out_temperature - temperature_max
        #    wasted_energy = (delta_T * ws_out_mass * PhysicsConfig.water_specific_heat_capacity)
        #    ws_out_temperature = temperature_max
        #ws_out_enthalpy = ws_in.enthalpy + thermal_energy_to_add
        #ws_in.change_slice_parameters(new_temperature=ws_out_temperature, new_enthalpy=ws_out_enthalpy, new_mass=ws_out_mass)
        #return ws_in, wasted_energy, thermal_energy_to_add


class HeatPumpController(cp.Component):
    """
    Heat Pump Controller. It takes data from other
    components and sends signal to the heat pump for
    activation or deactivation.

    Parameters
    --------------
    t_air_heating: float
        Minimum comfortable temperature for residents
    t_air_cooling: float
        Maximum comfortable temperature for residents
    offset: float
        Temperature offset to compensate the hysteresis
        correction for the building temperature change
    mode : int
        Mode index for operation type for this heat pump
    """
    # Inputs
    TemperatureMean = "Residence Temperature"
    ElectricityInput = "ElectricityInput"

    # Outputs
    State = "State"

    # Similar components to connect to:
    # 1. Building
    @utils.measure_execution_time
    def __init__(self,
                 my_simulation_parameters: SimulationParameters,
                 t_air_heating: float = 18.0,
                 t_air_cooling: float = 26.0,
                 offset: float = 0.0,
                 mode:int=1) -> None:
        super().__init__("HeatPumpController", my_simulation_parameters=my_simulation_parameters)
        self.build(t_air_cooling=t_air_cooling,
                   t_air_heating=t_air_heating,
                   offset=offset,
                   mode=mode)

        self.t_mC: cp.ComponentInput = self.add_input(self.component_name,
                                                      self.TemperatureMean,
                                                      LoadTypes.TEMPERATURE,
                                                      Units.CELSIUS,
                                                      True)
        self.electricity_inputC : cp.ComponentInput= self.add_input(self.component_name,
                                                                    self.ElectricityInput,
                                                                    LoadTypes.ELECTRICITY,
                                                                    Units.WATT,
                                                                    False)
        self.stateC: cp.ComponentOutput = self.add_output(self.component_name,
                                                          self.State,
                                                          LoadTypes.ANY,
                                                          Units.ANY)
        
        self.add_default_connections( Building, self.get_building_default_connections( ) )
        
    def get_building_default_connections( self ) -> List[cp.ComponentConnection]:
        log.information("setting building default connections in Heatpumpcontroller")
        connections = [ ]
        building_classname = Building.get_classname( )
        connections.append( cp.ComponentConnection( HeatPumpController.TemperatureMean, building_classname, Building.TemperatureMean ) )
        return connections

    def build(self, t_air_heating:float, t_air_cooling:float, offset:float, mode: float) -> None:
        # Sth
        self.controller_heatpumpmode = "off"
        self.previous_heatpump_mode = self.controller_heatpumpmode

        # Configuration
        self.t_set_heating = t_air_heating
        self.t_set_cooling = t_air_cooling
        self.offset = offset

        self.mode = mode
    def i_prepare_simulation(self) -> None:
        """ Prepares the simulation. """
        pass
    def i_save_state(self ) -> None:
        self.previous_heatpump_mode = self.controller_heatpumpmode

    def i_restore_state(self)  -> None:
        self.controller_heatpumpmode = self.previous_heatpump_mode

    def i_doublecheck(self, timestep: int, stsv: cp.SingleTimeStepValues)  -> None:
        pass
    def write_to_report(self) -> List[str]:
        lines = []
        lines.append("Heat Pump Controller")
        # todo: add more useful stuff here
        lines.append("tbd")
        return lines
    def i_simulate(self, timestep: int, stsv: cp.SingleTimeStepValues,  force_convergence: bool) -> None:
        # check demand, and change state of self.has_heating_demand, and self._has_cooling_demand
        if force_convergence:
            pass
        else:
            # Retrieves inputs
            t_m_old = stsv.get_input_value(self.t_mC)
            electricity_input = stsv.get_input_value(self.electricity_inputC)

            if self.mode == 1:
                self.conditions(t_m_old)
            elif self.mode == 2:
                self.smart_conditions(t_m_old, electricity_input)

        if self.controller_heatpumpmode == 'heating':
            state = 1
        if self.controller_heatpumpmode == 'cooling':
            state = -1
        if self.controller_heatpumpmode == 'off':
            state = 0
        stsv.set_output_value(self.stateC, state)

    def conditions(self, set_temp: float) -> None:
        maximum_heating_set_temp = self.t_set_heating + self.offset
        minimum_heating_set_temp = self.t_set_heating
        minimum_cooling_set_temp = self.t_set_cooling - self.offset
        maximum_cooling_set_temp = self.t_set_cooling

        if self.controller_heatpumpmode == 'heating':  # and daily_avg_temp < 15:
            if set_temp > maximum_heating_set_temp:  # 23
                self.controller_heatpumpmode = 'off'
                return
        if self.controller_heatpumpmode == 'cooling':
            if set_temp < minimum_cooling_set_temp: # 24
                self.controller_heatpumpmode = 'off'
                return
        if self.controller_heatpumpmode == 'off':
            #if pvs_surplus > ? and air_temp < minimum_heating_air + 2:
            if set_temp < minimum_heating_set_temp:  # 21
                self.controller_heatpumpmode = 'heating'
                return
            if set_temp > maximum_cooling_set_temp:  # 26
                self.controller_heatpumpmode = 'cooling'
                return

    def smart_conditions(self, set_temp: float, electricity_input: float) -> None:
        smart_offset_upper = 3
        smart_offset_lower = 0.5
        maximum_heating_set_temp = self.t_set_heating + self.offset
        if electricity_input < 0:
            maximum_heating_set_temp += smart_offset_upper
        # maximum_heating_set_temp = self.t_set_heating
        minimum_heating_set_temp = self.t_set_heating
        if electricity_input < 0:
            minimum_heating_set_temp += smart_offset_lower
        minimum_cooling_set_temp = self.t_set_cooling - self.offset
        # minimum_cooling_set_temp = self.t_set_cooling
        maximum_cooling_set_temp = self.t_set_cooling

        if self.controller_heatpumpmode == 'heating':  # and daily_avg_temp < 15:
            if set_temp > maximum_heating_set_temp:  # 23
                self.controller_heatpumpmode = 'off'
                return
        if self.controller_heatpumpmode == 'cooling':
            if set_temp < minimum_cooling_set_temp:  # 24
                self.controller_heatpumpmode = 'off'
                return
        if self.controller_heatpumpmode == 'off':
            # if pvs_surplus > ? and air_temp < minimum_heating_air + 2:
            if set_temp < minimum_heating_set_temp:  # 21
                self.controller_heatpumpmode = 'heating'
                return
            if set_temp > maximum_cooling_set_temp:  # 26
                self.controller_heatpumpmode = 'cooling'
                return

        #if timestep >= 60*24*30*3 and timestep <= 60*24*30*9:
        #    state = 0

        #log.information("Final state: {}\n".format(state))

    def prin1t_outpu1t(self, t_m:float, state: Any) -> None:
        log.information("==========================================")
        log.information("T m: {}".format(t_m))
        log.information("State: {}".format(state))




# -*- coding: utf-8 -*-
""" EconomicParameters Class will be written to a json file."""

from dataclasses import dataclass
from dataclasses_json import dataclass_json


@dataclass_json
@dataclass()
class EconomicParameters:

    """EconomicParameter Class is created."""

    insulation_bought: bool = True
    insulation_threshold: float = 1e3
    pv_bought: bool = True
    pv_threshold: float = 1e3  # in Watt
    smart_devices_bought: bool = True
    smart_devices_threshold: float = 5e2
    heatpump_bought: bool = False
    heatpump_threshold: float = 1e4
    buffer_bought: bool = True
    buffer_threshold: float = 1e2
    battery_bought: bool = True
    battery_threshold: float = 1e3  # in Wh
    h2system_bought: bool = False
    h2system_threshold: float = 5e3
    chp_bought: bool = False
    chp_threshold: float = 5e3
    electrolyzer_bought: bool = False
    electrolyzer_threshold: float = 5e3
    surpluscontroller_bought: bool = False
    surpluscontroller_threshold: float = 5e3
    ev_bought: bool = True
    ev_threshold: float = 2e4

# def create_economicparameters_file(
#         component: lt.ComponentType, capacity_unit: lt.Units, capacity_for_cost: List,
#         cost_per_capacity: List, time: List, costfactor_per_time: List, capacity_for_co2: List,
#         co2_per_capacity: List):

#     costfile = ComponentCost(
#         component=component, capacity_unit=capacity_unit, capacity_for_cost=capacity_for_cost,
#         cost_per_capacity=cost_per_capacity, time=time, costfactor_per_time=costfactor_per_time,
#         capacity_for_co2=capacity_for_co2, co2_per_capacity=co2_per_capacity)
#     costfile = json.dumps(asdict(costfile))

#     with open('ComponentCost' + component.value + '.json', 'w') as outfile:
#         outfile.write(hey)

# def write_batterycost_file():
#     create_componentcost_file(
#         component=lt.ComponentType.BATTERY, capacity_unit=lt.Units.WATT_HOUR, capacity_for_cost=[500,1000,2000,10000],
#         cost_per_capacity=[1000,1800,2500,5000], time=[2022,2030,2050], costfactor_per_time=[100,90,60],
#         capacity_for_co2=[1000, 10000], co2_per_capacity=[200, 2000] )
